@@ -166,9 +166,9 @@ void account_read_config_all(void)
 	}
 }
 
-void account_save_config_all(void)
+void account_write_config_all(void)
 {
-	prefs_account_save_config_all(account_list);
+	prefs_account_write_config_all(account_list);
 }
 
 PrefsAccount *account_find_from_smtp_server(const gchar *address,
@@ -419,7 +419,7 @@ void account_open(PrefsAccount *ac_prefs)
 		folderview_set_all();
 	}
 
-	account_save_config_all();
+	account_write_config_all();
 	account_set_menu();
 	main_window_reflect_prefs_all();
 }
@@ -762,6 +762,11 @@ static void account_delete(void)
 	row = GPOINTER_TO_INT(clist->selection->data);
 	ac_prefs = gtk_clist_get_row_data(clist, row);
 	if (ac_prefs->folder) {
+		FolderItem *item;
+
+		item = main_window_get()->summaryview->folder_item;
+		if (item && item->folder == FOLDER(ac_prefs->folder))
+			summary_clear_all(main_window_get()->summaryview);
 		folder_destroy(FOLDER(ac_prefs->folder));
 		folderview_set_all();
 	}
@@ -814,7 +819,7 @@ static void account_set_default(void)
 static void account_edit_close(void)
 {
 	account_list_set();
-	account_save_config_all();
+	account_write_config_all();
 
 	if (!cur_account && account_list) {
 		PrefsAccount *ac_prefs = (PrefsAccount *)account_list->data;
@@ -840,15 +845,17 @@ static gint account_delete_event(GtkWidget *widget, GdkEventAny *event,
 static void account_selected(GtkCList *clist, gint row, gint column,
 			     GdkEvent *event, gpointer data)
 {
-	if (event && event->type == GDK_2BUTTON_PRESS)
+	if (event && event->type == GDK_2BUTTON_PRESS) {
 		account_edit_prefs();
+		return;
+	}
 
 	if (column == COL_GETALL) {
 		PrefsAccount *ac;
 
 		ac = gtk_clist_get_row_data(clist, row);
-		if (ac->protocol == A_POP3 || ac->protocol == A_APOP ||
-		    ac->protocol == A_IMAP4 || ac->protocol == A_NNTP) {
+		if (ac->protocol == A_POP3 || ac->protocol == A_IMAP4 ||
+		    ac->protocol == A_NNTP) {
 			ac->recv_at_getall ^= TRUE;
 			account_clist_set_row(ac, row);
 		}
@@ -886,11 +893,6 @@ static gint account_clist_set_row(PrefsAccount *ac_prefs, gint row)
 			      "POP3 (SSL)" :
 			      ac_prefs->ssl_pop == SSL_STARTTLS ?
 			      "POP3 (TLS)" : "POP3") :
-			     ac_prefs->protocol == A_APOP ?
-			     (ac_prefs->ssl_pop == SSL_TUNNEL ?
-			      "POP3 (APOP, SSL)" :
-			      ac_prefs->ssl_pop == SSL_STARTTLS ?
-			      "POP3 (APOP, TLS)" : "POP3 (APOP)") :
 			     ac_prefs->protocol == A_IMAP4 ?
 			     (ac_prefs->ssl_imap == SSL_TUNNEL ?
 			      "IMAP4 (SSL)" :
@@ -902,7 +904,6 @@ static gint account_clist_set_row(PrefsAccount *ac_prefs, gint row)
 			     "";
 #else
 	text[COL_PROTOCOL] = ac_prefs->protocol == A_POP3  ? "POP3" :
-			     ac_prefs->protocol == A_APOP  ? "POP3 (APOP)" :
 			     ac_prefs->protocol == A_IMAP4 ? "IMAP4" :
 			     ac_prefs->protocol == A_NNTP  ? "NNTP" : "";
 #endif
@@ -920,7 +921,6 @@ static gint account_clist_set_row(PrefsAccount *ac_prefs, gint row)
 	}
 
 	has_getallbox = (ac_prefs->protocol == A_POP3  ||
-			 ac_prefs->protocol == A_APOP  ||
 			 ac_prefs->protocol == A_IMAP4 ||
 			 ac_prefs->protocol == A_NNTP);
 	getall = has_getallbox && ac_prefs->recv_at_getall;
