@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -571,6 +571,20 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 		gchar outbuf[BUFFSIZE];
 		gint len;
 		Base64Decoder *decoder;
+		gboolean uncanonicalize = FALSE;
+		FILE *tmpfp = outfp;
+
+		if (mimeinfo->mime_type == MIME_TEXT ||
+		    mimeinfo->mime_type == MIME_TEXT_HTML ||
+		    mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
+			uncanonicalize = TRUE;
+			tmpfp = my_tmpfile();
+			if (!tmpfp) {
+				perror("tmpfile");
+				if (tmp_file) fclose(outfp);
+				return NULL;
+			}
+		}
 
 		decoder = base64_decoder_new();
 		while (fgets(buf, sizeof(buf), infp) != NULL &&
@@ -581,9 +595,18 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 				g_warning("Bad BASE64 content\n");
 				break;
 			}
-			fwrite(outbuf, sizeof(gchar), len, outfp);
+			fwrite(outbuf, sizeof(gchar), len, tmpfp);
 		}
 		base64_decoder_free(decoder);
+
+		if (uncanonicalize) {
+			rewind(tmpfp);
+			while (fgets(buf, sizeof(buf), tmpfp) != NULL) {
+				strcrchomp(buf);
+				fputs(buf, outfp);
+			}
+			fclose(tmpfp);
+		}
 	} else if (mimeinfo->encoding_type == ENC_X_UUENCODE) {
 		gchar outbuf[BUFFSIZE];
 		gint len;
@@ -760,17 +783,17 @@ FILE *procmime_get_first_text_content(MsgInfo *msginfo)
 }
 
 gboolean procmime_find_string_part(MimeInfo *mimeinfo, const gchar *filename,
-				   const gchar *str, gboolean case_sens)
+				   const gchar *str, StrFindFunc find_func)
 {
 
 	FILE *infp, *outfp;
 	gchar buf[BUFFSIZE];
-	gchar *(* StrFindFunc) (const gchar *haystack, const gchar *needle);
 
 	g_return_val_if_fail(mimeinfo != NULL, FALSE);
 	g_return_val_if_fail(mimeinfo->mime_type == MIME_TEXT ||
 			     mimeinfo->mime_type == MIME_TEXT_HTML, FALSE);
 	g_return_val_if_fail(str != NULL, FALSE);
+	g_return_val_if_fail(find_func != NULL, FALSE);
 
 	if ((infp = fopen(filename, "rb")) == NULL) {
 		FILE_OP_ERROR(filename, "fopen");
@@ -783,13 +806,9 @@ gboolean procmime_find_string_part(MimeInfo *mimeinfo, const gchar *filename,
 	if (!outfp)
 		return FALSE;
 
-	if (case_sens)
-		StrFindFunc = strstr;
-	else
-		StrFindFunc = strcasestr;
-
 	while (fgets(buf, sizeof(buf), outfp) != NULL) {
-		if (StrFindFunc(buf, str) != NULL) {
+		strretchomp(buf);
+		if (find_func(buf, str)) {
 			fclose(outfp);
 			return TRUE;
 		}
@@ -801,7 +820,7 @@ gboolean procmime_find_string_part(MimeInfo *mimeinfo, const gchar *filename,
 }
 
 gboolean procmime_find_string(MsgInfo *msginfo, const gchar *str,
-			      gboolean case_sens)
+			      StrFindFunc find_func)
 {
 	MimeInfo *mimeinfo;
 	MimeInfo *partinfo;
@@ -810,6 +829,7 @@ gboolean procmime_find_string(MsgInfo *msginfo, const gchar *str,
 
 	g_return_val_if_fail(msginfo != NULL, FALSE);
 	g_return_val_if_fail(str != NULL, FALSE);
+	g_return_val_if_fail(find_func != NULL, FALSE);
 
 	filename = procmsg_get_message_file(msginfo);
 	if (!filename) return FALSE;
@@ -820,7 +840,7 @@ gboolean procmime_find_string(MsgInfo *msginfo, const gchar *str,
 		if (partinfo->mime_type == MIME_TEXT ||
 		    partinfo->mime_type == MIME_TEXT_HTML) {
 			if (procmime_find_string_part
-				(partinfo, filename, str, case_sens) == TRUE) {
+				(partinfo, filename, str, find_func) == TRUE) {
 				found = TRUE;
 				break;
 			}
@@ -1045,31 +1065,52 @@ EncodingType procmime_get_encoding_for_charset(const gchar *charset)
 		return ENC_8BIT;
 }
 
-EncodingType procmime_get_encoding_for_file(const gchar *file)
+EncodingType procmime_get_encoding_for_text_file(const gchar *file)
 {
 	FILE *fp;
-	guchar buf[BUFSIZ];
+	guchar buf[BUFFSIZE];
 	size_t len;
+	size_t octet_chars = 0;
+	size_t total_len = 0;
+	gfloat octet_percentage;
 
 	if ((fp = fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return ENC_UNKNOWN;
 	}
 
-	while ((len = fread(buf, sizeof(gchar), sizeof(buf), fp)) > 0) {
+	while ((len = fread(buf, sizeof(guchar), sizeof(buf), fp)) > 0) {
 		guchar *p;
 		gint i;
 
-		for (p = buf, i = 0; i < len; p++, i++) {
-			if (*p & 0x80) {
-				fclose(fp);
-				return ENC_BASE64;
-			}
+		for (p = buf, i = 0; i < len; ++p, ++i) {
+			if (*p & 0x80)
+				++octet_chars;
 		}
+		total_len += len;
 	}
 
 	fclose(fp);
-	return ENC_7BIT;
+
+	if (total_len > 0)
+		octet_percentage = (gfloat)octet_chars / (gfloat)total_len;
+	else
+		octet_percentage = 0.0;
+
+	debug_print("procmime_get_encoding_for_text_file(): "
+		    "8bit chars: %d / %d (%f%%)\n", octet_chars, total_len,
+		    100.0 * octet_percentage);
+
+	if (octet_percentage > 0.20) {
+		debug_print("using BASE64\n");
+		return ENC_BASE64;
+	} else if (octet_chars > 0) {
+		debug_print("using quoted-printable\n");
+		return ENC_QUOTED_PRINTABLE;
+	} else {
+		debug_print("using 7bit\n");
+		return ENC_7BIT;
+	}
 }
 
 const gchar *procmime_get_encoding_str(EncodingType encoding)
