@@ -98,6 +98,13 @@ static GdkColor good_sig_color = {
 	(gushort)0
 };
 
+static GdkColor untrusted_sig_color = {
+	(gulong)0,
+	(gushort)0xefff,
+	(gushort)0,
+	(gushort)0
+};
+
 static GdkColor nocheck_sig_color = {
 	(gulong)0,
 	(gushort)0,
@@ -509,6 +516,8 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		GdkColor *color;
 		if (!strcmp(mimeinfo->sigstatus, _("Good signature")))
 			color = &good_sig_color;
+        else if (!strcmp(mimeinfo->sigstatus, _("Valid signature (untrusted key)")))
+            color = &untrusted_sig_color;
 		else if (!strcmp(mimeinfo->sigstatus, _("BAD signature")))
 			color = &bad_sig_color;
 		else
@@ -1533,14 +1542,18 @@ static gint textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		if (summaryview)
 			summary_pass_key_press_event(summaryview, event);
 		else
-			textview_scroll_page(textview, FALSE);
+			textview_scroll_page
+				(textview,
+				 (event->state &
+				  (GDK_SHIFT_MASK|GDK_MOD1_MASK)) != 0);
 		break;
 	case GDK_BackSpace:
 		textview_scroll_page(textview, TRUE);
 		break;
 	case GDK_Return:
-		textview_scroll_one_line(textview,
-					 (event->state & GDK_MOD1_MASK) != 0);
+		textview_scroll_one_line
+			(textview, (event->state &
+				    (GDK_SHIFT_MASK|GDK_MOD1_MASK)) != 0);
 		break;
 	case GDK_Delete:
 		if (summaryview)
